@@ -33,13 +33,14 @@
 #include <stdio.h>
 
 static MicrofeedPublisher* instantiate_publisher(MicrofeedProvider* provider, const char* publisher_identifier, const char* directory, void* user_data);
-static MicrofeedFeed* instantiate_feed(MicrofeedPublisher* publisher, const char* uri, void* user_data);
+static MicrofeedFeed* instantiate_feed(MicrofeedPublisher* publisher, const char* uri, int create, void* user_data);
 static void no_more_publishers(MicrofeedProvider* publisher, void* user_data);
 static void initialize_settings(MicrofeedPublisher* publisher, void* user_data);
 static int update_setting(MicrofeedPublisher* publisher, const char* key, const char* value, void* user_data);
 static void* add_timeout(MicrofeedProvider* provider, unsigned long int interval, MicrofeedProviderTimeoutHandler handler, void* data, void* user_data);
 static void remove_timeout(MicrofeedProvider* provider, void* timeout_implementation, void* user_data);
-static MicrofeedError* download_image(MicrofeedFeed* feed, const char* uid, void** data, size_t* length, void* user_data);
+static MicrofeedError* download_image(MicrofeedPublisher* publisher, const char* url, void** data, size_t* length, void* user_data);
+static MicrofeedError* update_comments_feed(MicrofeedFeed* feed, int user_initiated, void* user_data);
 
 static MicrofeedProviderCallbacks provider_callbacks = {
 	.instantiate_publisher = instantiate_publisher,
@@ -52,10 +53,11 @@ static MicrofeedPublisherCallbacks publisher_callbacks = {
 	.instantiate_feed = instantiate_feed,
 	.initialize_settings = initialize_settings,
 	.update_setting = update_setting,
+	.store_data = download_image
 };
 
-static MicrofeedFeedCallbacks images_feed_callbacks = {
-	.download_item_data = download_image
+static MicrofeedFeedCallbacks comments_feed_callbacks = {
+	.update = update_comments_feed
 };
 
 int main(void) {
@@ -110,36 +112,35 @@ static MicrofeedPublisher* instantiate_publisher(MicrofeedProvider* provider, co
 	return publisher;
 }
 
-static MicrofeedFeed* instantiate_feed(MicrofeedPublisher* publisher, const char* uri, void* user_data) {
+static MicrofeedFeed* instantiate_feed(MicrofeedPublisher* publisher, const char* uri, int create, void* user_data) {
 	MicrofeedFeed* feed = NULL;
 	MicrofeedMain* microfeed_main;
 	size_t length;
 	char* url;
-	MicrofeedFeed* images_feed;
+	MicrofeedFeed* master_feed;
 	
 	microfeed_main = (MicrofeedMain*)user_data;
 
 	if (!strcmp(uri, MICROFEED_FEED_URI_OVERVIEW)) {
 		url = strdup("/statuses/friends_timeline.json");
-		feed = microfeed_feed_new(publisher, uri, "Overview", MICROFEED_FEED_PERMISSION_ADD, &overview_feed_callbacks, url);
-		images_feed = microfeed_publisher_get_or_instantiate_feed(publisher, MICROFEED_FEED_URI_IMAGES);
-		microfeed_feed_add_item_data_property(feed, MICROFEED_ITEM_PROPERTY_NAME_USER_IMAGE, images_feed);
-		microfeed_feed_add_item_data_property(feed, MICROFEED_ITEM_PROPERTY_NAME_CONTENT_IMAGE, images_feed);
-		microfeed_object_unref(images_feed, MicrofeedFeed);
+		feed = microfeed_feed_new(publisher, uri, "Overview", MICROFEED_FEED_PERMISSION_ADD, 1, &overview_feed_callbacks, url);
+		microfeed_feed_add_data_property(feed, MICROFEED_ITEM_PROPERTY_NAME_USER_IMAGE);
+		microfeed_feed_add_data_property(feed, MICROFEED_ITEM_PROPERTY_NAME_CONTENT_IMAGE);
+		microfeed_feed_add_virtual_feed_property(feed, ".twitter.id", MICROFEED_FEED_URI_USER_PREFIX, 1);
+		microfeed_feed_add_virtual_feed_property(feed, MICROFEED_ITEM_PROPERTY_NAME_REFERRED_ITEM, MICROFEED_FEED_URI_COMMENTS_PREFIX, 1);
+		microfeed_feed_add_virtual_feed_property(feed, NULL, MICROFEED_FEED_URI_COMMENTS_PREFIX, 0);
 	} else if (!strcmp(uri, MICROFEED_FEED_URI_CONTACTS)) {
-		feed = microfeed_feed_new(publisher, uri, "Contacts", MICROFEED_FEED_PERMISSION_ADD, &contacts_callbacks, NULL);
-		images_feed = microfeed_publisher_get_or_instantiate_feed(publisher, MICROFEED_FEED_URI_IMAGES);
-		microfeed_feed_add_item_data_property(feed, MICROFEED_ITEM_PROPERTY_NAME_USER_IMAGE, images_feed);
-		microfeed_object_unref(images_feed, MicrofeedFeed);
-	} else if (!strcmp(uri, MICROFEED_FEED_URI_IMAGES)) {
-		feed = microfeed_feed_new(publisher, uri, "Images", MICROFEED_FEED_PERMISSION_NONE, &images_feed_callbacks, NULL);
+		feed = microfeed_feed_new(publisher, uri, "Contacts", MICROFEED_FEED_PERMISSION_ADD, 1, &contacts_callbacks, NULL);
+		microfeed_feed_add_data_property(feed, MICROFEED_ITEM_PROPERTY_NAME_USER_IMAGE);
 	} else if ((length = microfeed_util_string_starts_with(uri, MICROFEED_FEED_URI_USER_PREFIX))) {
-		url = microfeed_util_string_concatenate("/statuses/user_timeline.json?user_id=", uri + strlen(MICROFEED_FEED_URI_USER_PREFIX), NULL);
-		feed = microfeed_feed_new(publisher, uri, uri + length, MICROFEED_FEED_PERMISSION_NONE, &feed_callbacks, url);
-		images_feed = microfeed_publisher_get_or_instantiate_feed(publisher, MICROFEED_FEED_URI_IMAGES);
-		microfeed_feed_add_item_data_property(feed, MICROFEED_ITEM_PROPERTY_NAME_USER_IMAGE, images_feed);
-		microfeed_feed_add_item_data_property(feed, MICROFEED_ITEM_PROPERTY_NAME_CONTENT_IMAGE, images_feed);
-		microfeed_object_unref(images_feed, MicrofeedFeed);
+		url = microfeed_util_string_concatenate("/statuses/user_timeline.json?user_id=", uri + length, NULL);
+		master_feed = microfeed_publisher_get_or_instantiate_feed(publisher, MICROFEED_FEED_URI_OVERVIEW, 1);
+		feed = microfeed_feed_new_virtual(master_feed, uri, uri + length, MICROFEED_FEED_PERMISSION_NONE, create, &feed_callbacks, url);
+		microfeed_object_unref(master_feed, MicrofeedFeed);
+	} else if ((length = microfeed_util_string_starts_with(uri, MICROFEED_FEED_URI_COMMENTS_PREFIX))) {
+		master_feed = microfeed_publisher_get_or_instantiate_feed(publisher, MICROFEED_FEED_URI_OVERVIEW, 1);
+		feed = microfeed_feed_new_virtual(master_feed, uri, uri + length, MICROFEED_FEED_PERMISSION_NONE, create, &comments_feed_callbacks, NULL);
+		microfeed_object_unref(master_feed, MicrofeedFeed);
 	}
 
 	return feed;	
@@ -155,13 +156,13 @@ static int update_setting(MicrofeedPublisher* publisher, const char* key, const
 	return 1;
 }
 
-static MicrofeedError* download_image(MicrofeedFeed* feed, const char* uid, void** data, size_t* length, void* user_data) {
+static MicrofeedError* download_image(MicrofeedPublisher* publisher, const char* url, void** data, size_t* length, void* user_data) {
 	char* buffer;
 	MicrofeedError* error = NULL;
 	MicrofeedHttp* http;
 
 	http = microfeed_http_new();
-	if ((buffer = (void*)microfeed_http_get_data(http, uid, length))) {
+	if ((buffer = (void*)microfeed_http_get_data(http, url, length))) {
 		*data = malloc(*length);
 		memcpy(*data, buffer, *length);
 	} else {
@@ -214,3 +215,70 @@ static void remove_timeout(MicrofeedProvider* provider, void* timeout_implementa
 	microfeed_main_remove_timeout(microfeed_main, timeout_data->timeout);	
 	microfeed_memory_free(timeout_data);
 }
+
+static MicrofeedError* update_comments_feed(MicrofeedFeed* feed, int user_initiated, void* user_data) {
+	MicrofeedError* error = NULL;
+	const char* uri;
+	size_t length;
+	char* site;
+	char* url;
+	MicrofeedHttp* http;
+	int rate_limit;
+	time_t delta = 0;
+	MicrofeedJson* status;
+	const char* id;
+	const char* created_at;
+	time_t timestamp;
+	MicrofeedItem* item;
+	const char* cs;
+	char* s;
+	
+	uri = microfeed_feed_get_uri(feed);
+	if ((length = microfeed_util_string_starts_with(uri, MICROFEED_FEED_URI_COMMENTS_PREFIX))) {
+		site = microfeed_publisher_get_setting_value(microfeed_feed_get_publisher(feed), "service.url", "http://twitter.com");
+		url = microfeed_util_string_concatenate(site, "/statuses/show/", uri + length, ".json", NULL);
+		free(site);
+
+		http = microfeed_http_new();
+		do_authentication(microfeed_feed_get_publisher(feed), http);
+		if ((rate_limit = get_rate_limit(microfeed_feed_get_publisher(feed), http)) != -1 && rate_limit < 1) {
+			error = microfeed_error_new(MICROFEED_ERROR_FROM_SERVICE, "Rate limit exceeded."); /* TODO: Change to MICROFEED_ERROR_SERVICE_RATE_LIMIT_EXCEEDED */
+		} else if ((status = microfeed_http_get_json(http, url))) {
+			if ((id = microfeed_json_get_as_string(status, "id"))) {
+				if (microfeed_http_get_server_time(http)) {
+					delta = microfeed_http_get_server_time(http) - microfeed_http_get_reply_start_time(http);
+				}
+
+			/* TODO: ? */
+			delta = 0;
+			
+				if ((created_at = microfeed_json_get_string(status, "created_at"))) {
+					timestamp = microfeed_http_parse_date(http, created_at) - delta;
+				} else {
+					timestamp = time(NULL) - delta;
+				}
+				item = microfeed_item_new_with_status(id, timestamp, MICROFEED_ITEM_STATUS_NEW);
+				microfeed_item_set_property(item, MICROFEED_ITEM_PROPERTY_NAME_CONTENT_TEXT, microfeed_json_get_string(status, "text"));
+				microfeed_item_set_property(item, MICROFEED_ITEM_PROPERTY_NAME_USER_NICK, microfeed_json_get_string_by_path(status, "user", "screen_name", NULL));
+				if ((cs = microfeed_json_get_as_string_by_path(status, "user", "id", NULL))) {
+					microfeed_item_set_property(item, ".twitter.id", cs);						
+					s = microfeed_util_string_concatenate(MICROFEED_FEED_URI_USER_PREFIX, cs, NULL);
+					microfeed_item_set_property(item, MICROFEED_ITEM_PROPERTY_NAME_USER_FEED, s);
+					free(s);
+				}
+				microfeed_item_set_property(item, MICROFEED_ITEM_PROPERTY_NAME_USER_IMAGE, microfeed_json_get_string_by_path(status, "user", "profile_image_url", NULL));
+				if (!microfeed_json_is_null(status, "in_reply_to_status_id")) {
+					microfeed_item_set_property(item, MICROFEED_ITEM_PROPERTY_NAME_REFERRED_ITEM, microfeed_json_get_as_string(status, "in_reply_to_status_id"));
+				}
+
+				microfeed_feed_replace_item(feed, item);
+				microfeed_item_free(item);
+			}
+			microfeed_json_free(status);
+		}
+		microfeed_http_free(http);
+		free(url);
+	}
+	
+	return error;
+}
